'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const config = require('@boost/config');

const constants = require('./constants.js');

class Config extends config.Configuration {
  blueprint(schemas, onConstruction) {
    const bool = schemas.bool,
          number = schemas.number,
          object = schemas.object,
          shape = schemas.shape,
          string = schemas.string;
    const moduleSchema = string(process.env.BEEMO_CONFIG_MODULE);
    return {
      configure: shape({
        cleanup: bool(false),
        parallel: bool(true)
      }),
      debug: bool(),
      drivers: config.createPluginsSchema(schemas),
      execute: shape({
        concurrency: number(3).gt(0),
        graph: bool(true),
        output: string().oneOf(['', constants.STRATEGY_BUFFER, constants.STRATEGY_PIPE, constants.STRATEGY_STREAM, constants.STRATEGY_NONE])
      }),
      module: onConstruction ? moduleSchema : moduleSchema.required().notEmpty(),
      scripts: config.createPluginsSchema(schemas),
      settings: object()
    };
  }

  bootstrap() {
    this.configureFinder({
      errorIfNoRootFound: true
    });
    this.addProcessHandler('drivers', config.mergePlugins);
    this.addProcessHandler('scripts', config.mergePlugins);
  }

}

exports.Config = Config;
//# sourceMappingURL=Config.js.map
