import ts from 'typescript';
import { DriverOptions } from '@beemo/core';
export declare type TargetSetting = 'es3' | 'es5' | 'es6' | 'es2015' | 'es2016' | 'es2017' | 'es2018' | 'es2019' | 'es2020' | 'es2021' | 'es2022' | 'esnext' | 'json' | 'latest';
export declare type ModuleSetting = 'amd' | 'commonjs' | 'es6' | 'es2015' | 'es2020' | 'es2022' | 'esnext' | 'node12' | 'nodenext' | 'none' | 'system' | 'umd';
export declare type ModuleResolutionSetting = 'classic' | 'node' | 'node12' | 'nodenext';
export declare type LibSetting = 'dom.iterable' | 'dom' | 'es5' | 'es6' | 'es7' | 'es2015.collection' | 'es2015.core' | 'es2015.generator' | 'es2015.iterable' | 'es2015.promise' | 'es2015.proxy' | 'es2015.reflect' | 'es2015.symbol.wellknown' | 'es2015.symbol' | 'es2015' | 'es2016.array.include' | 'es2016.full' | 'es2016' | 'es2017.full' | 'es2017.intl' | 'es2017.object' | 'es2017.sharedmemory' | 'es2017.string' | 'es2017.typedarrays' | 'es2017' | 'es2018.asyncgenerator' | 'es2018.asynciterable' | 'es2018.full' | 'es2018.intl' | 'es2018.promise' | 'es2018.regexp' | 'es2018' | 'es2019.array' | 'es2019.full' | 'es2019.object' | 'es2019.string' | 'es2019.symbol' | 'es2019' | 'es2020.bigint' | 'es2020.full' | 'es2020.intl' | 'es2020.promise' | 'es2020.sharedmemory' | 'es2020.string' | 'es2020.symbol.wellknown' | 'es2020' | 'es2021.promise' | 'es2021.string' | 'es2021.weakref' | 'es2021' | 'es2022' | 'esnext.array' | 'esnext.asynciterable' | 'esnext.full' | 'esnext.intl' | 'esnext.promise' | 'esnext.string' | 'esnext.weakref' | 'esnext' | 'scripthost' | 'webworker.importscripts' | 'webworker.iterable' | 'webworker';
export declare type JSXSetting = 'none' | 'preserve' | 'react-jsx-dev' | 'react-jsx' | 'react-native' | 'react';
export declare type NewLineSetting = 'crlf' | 'lf';
export interface CompilerOptions {
    allowJs?: boolean;
    allowSyntheticDefaultImports?: boolean;
    allowUmdGlobalAccess?: boolean;
    allowUnreachableCode?: boolean;
    allowUnusedLabels?: boolean;
    alwaysStrict?: boolean;
    assumeChangesOnlyAffectDirectDependencies?: boolean;
    baseUrl?: string;
    charset?: string;
    checkJs?: boolean;
    composite?: boolean;
    declaration?: boolean;
    declarationDir?: string;
    declarationMap?: boolean;
    disableReferencedProjectLoad?: boolean;
    disableSizeLimit?: boolean;
    disableSolutionSearching?: boolean;
    disableSourceOfProjectReferenceRedirect?: boolean;
    downlevelIteration?: boolean;
    emitBOM?: boolean;
    emitDeclarationOnly?: boolean;
    emitDecoratorMetadata?: boolean;
    esModuleInterop?: boolean;
    exactOptionalPropertyTypes?: boolean;
    experimentalDecorators?: boolean;
    forceConsistentCasingInFileNames?: boolean;
    importHelpers?: boolean;
    importsNotUsedAsValues?: 'error' | 'preserve' | 'remove';
    incremental?: boolean;
    inlineSourceMap?: boolean;
    inlineSources?: boolean;
    isolatedModules?: boolean;
    jsx?: JSXSetting;
    jsxFactory?: string;
    jsxFragmentFactory?: string;
    jsxImportSource?: string;
    keyofStringsOnly?: boolean;
    lib?: LibSetting[];
    locale?: string;
    mapRoot?: string;
    maxNodeModuleJsDepth?: number;
    module?: ModuleSetting;
    moduleResolution?: ModuleResolutionSetting;
    newLine?: NewLineSetting;
    noEmit?: boolean;
    noEmitHelpers?: boolean;
    noEmitOnError?: boolean;
    noErrorTruncation?: boolean;
    noFallthroughCasesInSwitch?: boolean;
    noImplicitAny?: boolean;
    noImplicitOverride?: boolean;
    noImplicitReturns?: boolean;
    noImplicitThis?: boolean;
    noImplicitUseStrict?: boolean;
    noLib?: boolean;
    noResolve?: boolean;
    noStrictGenericChecks?: boolean;
    noUncheckedIndexedAccess?: boolean;
    noUnusedLocals?: boolean;
    noUnusedParameters?: boolean;
    out?: string;
    outDir?: string;
    outFile?: string;
    paths?: Record<string, string[]>;
    preserveConstEnums?: boolean;
    preserveSymlinks?: boolean;
    preserveValueImports?: boolean;
    pretty?: boolean;
    project?: string;
    reactNamespace?: string;
    removeComments?: boolean;
    resolveJsonModule?: boolean;
    rootDir?: string;
    rootDirs?: string[];
    skipDefaultLibCheck?: boolean;
    skipLibCheck?: boolean;
    sourceMap?: boolean;
    sourceRoot?: string;
    strict?: boolean;
    strictBindCallApply?: boolean;
    strictFunctionTypes?: boolean;
    strictNullChecks?: boolean;
    strictPropertyInitialization?: boolean;
    stripInternal?: boolean;
    suppressExcessPropertyErrors?: boolean;
    suppressImplicitAnyIndexErrors?: boolean;
    target?: TargetSetting;
    traceResolution?: boolean;
    tsBuildInfoFile?: string;
    typeRoots?: string[];
    types?: string[];
    useDefineForClassFields?: boolean;
    useUnknownInCatchVariables?: boolean;
}
export interface TypeScriptConfig {
    compileOnSave?: boolean;
    compilerOptions?: CompilerOptions;
    exclude?: string[];
    extends?: string;
    files?: string[];
    include?: string[];
    references?: ts.ProjectReference[];
    typeAcquisition?: ts.TypeAcquisition;
}
export interface TypeScriptArgs {
    all?: boolean;
    allowJs?: boolean;
    allowSyntheticDefaultImports?: boolean;
    allowUmdGlobalAccess?: boolean;
    allowUnreachableCode?: boolean;
    allowUnusedLabels?: boolean;
    alwaysStrict?: boolean;
    assumeChangesOnlyAffectDirectDependencies?: boolean;
    b?: boolean;
    baseUrl?: string;
    build?: boolean;
    charset?: string;
    checkJs?: boolean;
    composite?: boolean;
    d?: boolean;
    declaration?: boolean;
    declarationDir?: string;
    declarationMap?: boolean;
    diagnostics?: boolean;
    disableReferencedProjectLoad?: boolean;
    disableSizeLimit?: boolean;
    disableSolutionSearching?: boolean;
    disableSourceOfProjectReferenceRedirect?: boolean;
    downlevelIteration?: boolean;
    emitBOM?: boolean;
    emitDeclarationOnly?: boolean;
    emitDecoratorMetadata?: boolean;
    esModuleInterop?: boolean;
    exactOptionalPropertyTypes?: boolean;
    experimentalDecorators?: boolean;
    extendedDiagnostics?: boolean;
    forceConsistentCasingInFileNames?: boolean;
    generateCpuProfile?: string;
    generateTrace?: string;
    h?: boolean;
    help?: boolean;
    i?: boolean;
    importHelpers?: boolean;
    importsNotUsedAsValues?: boolean;
    incremental?: boolean;
    init?: boolean;
    inlineSourceMap?: boolean;
    inlineSources?: boolean;
    isolatedModules?: boolean;
    jsx?: JSXSetting;
    jsxFactory?: string;
    jsxFragmentFactory?: string;
    jsxImportSource?: string;
    keyofStringsOnly?: boolean;
    lib?: LibSetting | LibSetting[];
    listEmittedFiles?: number;
    listFiles?: number;
    listFilesOnly?: number;
    locale?: string;
    m?: ModuleSetting;
    mapRoot?: string;
    maxNodeModuleJsDepth?: number;
    module?: ModuleSetting;
    moduleResolution?: ModuleResolutionSetting;
    newLine?: NewLineSetting;
    noEmit?: boolean;
    noEmitHelpers?: boolean;
    noEmitOnError?: boolean;
    noErrorTruncation?: boolean;
    noFallthroughCasesInSwitch?: boolean;
    noImplicitAny?: boolean;
    noImplicitOverride?: boolean;
    noImplicitReturns?: boolean;
    noImplicitThis?: boolean;
    noImplicitUseStrict?: boolean;
    noLib?: boolean;
    noResolve?: boolean;
    noStrictGenericChecks?: boolean;
    noUncheckedIndexedAccess?: boolean;
    noUnusedLocals?: boolean;
    noUnusedParameters?: boolean;
    out?: string;
    outDir?: string;
    outFile?: string;
    p?: string;
    paths?: string[];
    plugins?: string[];
    preserveConstEnums?: boolean;
    preserveSymlinks?: boolean;
    preserveValueImports?: boolean;
    preserveWatchOutput?: boolean;
    pretty?: boolean;
    project?: string;
    reactNamespace?: string;
    removeComments?: boolean;
    resolveJsonModule?: boolean;
    rootDir?: string;
    rootDirs?: string[];
    showConfig?: number;
    skipDefaultLibCheck?: boolean;
    skipLibCheck?: boolean;
    sourceMap?: boolean;
    sourceRoot?: string;
    strict?: boolean;
    strictBindCallApply?: boolean;
    strictFunctionTypes?: boolean;
    strictNullChecks?: boolean;
    strictPropertyInitialization?: boolean;
    stripInternal?: boolean;
    suppressExcessPropertyErrors?: object;
    suppressImplicitAnyIndexErrors?: boolean;
    t?: TargetSetting;
    target?: TargetSetting;
    traceResolution?: boolean;
    tsBuildInfoFile?: string;
    typeRoots?: string[];
    types?: string[];
    useDefineForClassFields?: boolean;
    useUnknownInCatchVariables?: boolean;
    v?: boolean;
    version?: boolean;
    w?: boolean;
    watch?: boolean;
}
export interface TypeScriptOptions extends DriverOptions {
    buildFolder?: string;
    declarationOnly?: boolean;
    devFolders?: string[];
    globalTypes?: boolean;
    localTypes?: boolean;
    srcFolder?: string;
    testsFolder?: string;
    typesFolder?: string;
}
//# sourceMappingURL=types.d.ts.map