'use strict';

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

Object.defineProperty(exports, '__esModule', {
  value: true
});

const rimraf = require('rimraf');

const core = require('@beemo/core');

const event = require('@boost/event');

const syncProjectRefs = require('./commands/syncProjectRefs.js');

const helpers = require('./helpers.js');

const _interopDefault = e => e && e.__esModule ? e : {
  default: e
};

const rimraf__default = /*#__PURE__*/_interopDefault(rimraf);
/* eslint-disable no-param-reassign */
// Success: Writes nothing to stdout or stderr
// Failure: Writes to stdout on syntax and type error


class TypeScriptDriver extends core.Driver {
  constructor(...args) {
    super(...args);
    this.name = '@beemo/driver-typescript';
    this.onCreateProjectConfigFile = new event.Event('create-project-config-file');

    this.handleCleanTarget = context => {
      var _this$config$compiler;

      // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
      const outDir = context.getRiskyOption('outDir', true) || ((_this$config$compiler = this.config.compilerOptions) === null || _this$config$compiler === void 0 ? void 0 : _this$config$compiler.outDir);

      if (context.getRiskyOption('clean') && typeof outDir === 'string' && outDir) {
        rimraf__default.default.sync(core.Path.resolve(outDir).path());
      }

      return Promise.resolve();
    };

    this.handlePrepareConfigs = (context, configPath, config) => {
      const tool = this.tool;
      const _this$options = this.options,
            _this$options$devFold = _this$options.devFolders,
            devFolders = _this$options$devFold === void 0 ? [] : _this$options$devFold,
            srcFolder = _this$options.srcFolder,
            testsFolder = _this$options.testsFolder;
      const workspacePackages = tool.project.getWorkspacePackages();

      if (workspacePackages.length === 0) {
        return;
      } // Extract compiler optionst to a separate config


      const optionsConfigPath = configPath.parent().append('tsconfig.options.json');
      void helpers.writeFile(optionsConfigPath, {
        compilerOptions: _objectSpread(_objectSpread({}, config.compilerOptions), {}, {
          composite: true,
          declaration: true,
          declarationMap: true,
          outDir: undefined,
          outFile: undefined
        })
      }); // Delete problematic root options

      delete config.compilerOptions;
      delete config.include;
      delete config.exclude; // Generate references and update paths

      config.extends = './tsconfig.options.json';
      config.files = [];
      config.references || (config.references = []);
      workspacePackages.forEach(({
        metadata
      }) => {
        const pkgPath = new core.Path(metadata.packagePath);
        const refPaths = [];

        if (srcFolder && pkgPath.append(srcFolder).exists()) {
          refPaths.push(pkgPath);
        }

        if (testsFolder && pkgPath.append(testsFolder).exists()) {
          refPaths.push(pkgPath.append(testsFolder));
        }

        devFolders.forEach(devFolder => {
          if (devFolder && pkgPath.append(devFolder).exists()) {
            refPaths.push(pkgPath.append(devFolder));
          }
        });
        refPaths.forEach(refPath => {
          config.references.push({
            path: helpers.join(tool.project.root.relativeTo(refPath))
          });
        });
      }); // Add to context so that it can be automatically cleaned up

      context.addConfigPath('typescript', optionsConfigPath);
    };
  }

  blueprint(schemas) {
    const array = schemas.array,
          bool = schemas.bool,
          string = schemas.string;
    return _objectSpread(_objectSpread({}, super.blueprint(schemas)), {}, {
      buildFolder: string('lib'),
      declarationOnly: bool(),
      devFolders: array().of(string()),
      globalTypes: bool(true),
      localTypes: bool(true),
      srcFolder: string('src'),
      testsFolder: string('tests'),
      typesFolder: string('types')
    });
  }

  bootstrap() {
    this.setMetadata({
      bin: 'tsc',
      commandOptions: {
        clean: {
          default: false,
          description: this.tool.msg('app:typescriptCleanOption'),
          type: 'boolean'
        }
      },
      configName: 'tsconfig.json',
      configOption: '',
      description: this.tool.msg('app:typescriptDescription'),
      helpOption: '--help --all',
      title: 'TypeScript',
      watchOptions: ['-w', '--watch'],
      workspaceStrategy: 'copy'
    });
    this.registerCommand('sync-project-refs', {
      description: this.tool.msg('app:typescriptSyncProjectRefsDescription')
    }, syncProjectRefs.syncProjectRefs);
    this.onCreateConfigFile.listen(this.handlePrepareConfigs);
    this.onBeforeExecute.listen(this.handleCleanTarget);
  }
  /**
   * Automatically clean the target folder if `outDir` and `--clean` is used.
   */


}

exports.TypeScriptDriver = TypeScriptDriver;
//# sourceMappingURL=TypeScriptDriver.js.map
