'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function mapToStringList(value) {
  if (!Array.isArray(value)) {
    return [String(value)];
  }

  return value.map(String);
}
/**
 * Format a parsed arguments object back into a command line list of arguments.
 */


function format(argument) {
  const _argument$command = argument.command,
        command = _argument$command === void 0 ? [] : _argument$command,
        _argument$options = argument.options,
        options = _argument$options === void 0 ? {} : _argument$options,
        _argument$params = argument.params,
        params = _argument$params === void 0 ? [] : _argument$params,
        _argument$rest = argument.rest,
        rest = _argument$rest === void 0 ? [] : _argument$rest;
  const args = []; // Commands

  if (command.length > 0) {
    args.push(command.join(':'));
  } // Options


  Object.entries(options).forEach(([name, value]) => {
    if (typeof value === 'boolean') {
      args.push(`--${!value ? 'no-' : ''}${name}`);
    } else {
      args.push(`--${name}`, ...mapToStringList(value));
    }
  }); // Params

  if (params.length > 0) {
    args.push(...mapToStringList(params));
  } // Rest


  if (rest.length > 0) {
    args.push('--', ...mapToStringList(rest));
  }

  return args;
}

exports.format = format;
//# sourceMappingURL=format.js.map
