'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const ArgsError = require('./ArgsError.js');

const parse = require('./parse.js');
/**
 * Parse a list of command line arguments (typically from `process.argv`) into an arguments
 * object using a context factory. The factory can customize the parser options based on the
 * arguments being parsed.
 */


function parseInContext(argv, context) {
  let options; // Loop through each arg until we find a context

  argv.some(arg => {
    options = context(arg, argv);
    return !!options;
  }); // Fail if context not found

  if (!options) {
    throw new ArgsError.ArgsError('CONTEXT_REQUIRED');
  }

  return parse.parse(argv, options);
}

exports.parseInContext = parseInContext;
//# sourceMappingURL=parseInContext.js.map
