'use strict';

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

Object.defineProperty(exports, '__esModule', {
  value: true
});

const _rollupPluginBabelHelpers = require('./_virtual/_rollupPluginBabelHelpers.js');

const execa = require('execa');

const common = require('@boost/common');

const CLIError = require('./CLIError.js');

const CommandManager = require('./CommandManager.js');

const constants = require('./constants.js');

const mapCommandMetadata = require('./helpers/mapCommandMetadata.js');

const getConstructor = require('./metadata/getConstructor.js');

const getInheritedCategories = require('./metadata/getInheritedCategories.js');

const getInheritedOptions = require('./metadata/getInheritedOptions.js');

const globalOptions = require('./metadata/globalOptions.js');

const validateConfig = require('./metadata/validateConfig.js');

const validateOptions = require('./metadata/validateOptions.js');

const validateParams = require('./metadata/validateParams.js');

const _interopDefault = e => e && e.__esModule ? e : {
  default: e
};

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  const n = Object.create(null);

  if (e) {
    for (const k in e) {
      if (k !== 'default') {
        const d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: () => e[k]
        });
      }
    }
  }

  n.default = e;
  return Object.freeze(n);
}

const execa__default = /*#__PURE__*/_interopDefault(execa);

var _dec, _class, _class2, _temp;

let Command = (_dec = common.Memoize(), (_class = (_temp = _class2 = class Command extends CommandManager.CommandManager {
  // Args

  /** Value of `--help`, `-h` passed on the command line. */

  /** Value of `--locale` passed on the command line. */

  /** Additional arguments passed after `--` on the command line. */

  /** Unknown options passed on the command line. */

  /** Value of `--version`, `-v` passed on the command line. */
  // Methods

  /** Method to exit the current program, with an optional error code. */

  /** Method to log to the console using a log level. */
  // Internals

  /** @internal */

  /** @internal */

  /** @internal */
  constructor(options) {
    super(options);
    this.help = false;
    this.locale = 'en';
    this.rest = [];
    this.unknown = {};
    this.version = false;
    this.exit = void 0;
    this.log = void 0;
    this[constants.INTERNAL_OPTIONS] = void 0;
    this[constants.INTERNAL_PARAMS] = void 0;
    this[constants.INTERNAL_PROGRAM] = void 0;

    this.runProgram = argv => this.getProgram().run(argv, undefined, true);

    this.runTask = (task, ...args) => {
      // We dont want tasks to have full access to the command
      // and its methods, so recreate a similar but smaller context.
      const context = _objectSpread(_objectSpread({
        exit: this.exit,
        log: this.log,
        rest: this.rest,
        unknown: this.unknown
      }, this[constants.INTERNAL_OPTIONS]), {}, {
        runProgram: this.runProgram,
        runTask: this.runTask
      });

      return task.apply(context, args);
    };

    this.handleBeforeRegister = subPath => {
      const path = this.getPath();

      if (!subPath.startsWith(path)) {
        throw new CLIError.CLIError('COMMAND_INVALID_SUBPATH', [subPath, path]);
      }
    };

    this.onBeforeRegister.listen(this.handleBeforeRegister);
  }
  /**
   * Validate options passed to the constructor.
   */
  // Empty blueprint so that sub-classes may type correctly


  blueprint(schemas) {
    return {};
  }
  /**
   * Create a React element based on the Help component.
   */


  async createHelp() {
    var _this$constants$INTER;

    const metadata = this.getMetadata();

    const _await$Promise$resolv = await Promise.resolve().then(() => /*#__PURE__*/_interopNamespace(require('react'))),
          createElement = _await$Promise$resolv.createElement;

    const _await$Promise$resolv2 = await Promise.resolve().then(() => require('./components/Help.js')),
          Help = _await$Promise$resolv2.Help;

    return createElement(Help, {
      categories: metadata.categories,
      commands: mapCommandMetadata.mapCommandMetadata(metadata.commands),
      config: metadata,
      delimiter: (_this$constants$INTER = this[constants.INTERNAL_PROGRAM]) === null || _this$constants$INTER === void 0 ? void 0 : _this$constants$INTER.options.delimiter,
      header: metadata.path,
      options: metadata.options,
      params: metadata.params
    });
  }
  /**
   * Execute a system native command with the given arguments
   * and pass the results through a promise. This does *not* execute Boost CLI
   * commands, use `runProgram()` instead.
   */


  executeCommand(command, args, options = {})
  /* infer */
  {
    const _this$getProgram = this.getProgram(),
          streams = _this$getProgram.streams;

    return execa__default.default(command, args, _objectSpread(_objectSpread({}, streams), options));
  }
  /**
   * Return the current command class as an arguments object.
   * Options and params must be parsed first to operate correctly.
   */


  getArguments() {
    var _this$constants$INTER2, _this$constants$INTER3;

    return {
      command: this.getPath().split(':'),
      errors: [],
      options: (_this$constants$INTER2 = this[constants.INTERNAL_OPTIONS]) !== null && _this$constants$INTER2 !== void 0 ? _this$constants$INTER2 : {},
      params: (_this$constants$INTER3 = this[constants.INTERNAL_PARAMS]) !== null && _this$constants$INTER3 !== void 0 ? _this$constants$INTER3 : [],
      rest: this.rest,
      unknown: this.unknown
    };
  }
  /**
   * Validate and return all metadata registered to this command instance.
   */


  getMetadata() {
    this.initializeAndValidate();
    const ctor = getConstructor.getConstructor(this);
    const options = {}; // Since default values for options are represented as class properties,
    // we need to inject the defaults into the argument parsing layer.
    // We can easily do this here and avoid a lot of headache.

    Object.entries(getInheritedOptions.getInheritedOptions(this)).forEach(([option, config]) => {
      var _this$option;

      options[option] = _objectSpread(_objectSpread({}, config), {}, {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        default: (_this$option = this[option]) !== null && _this$option !== void 0 ? _this$option : config.default
      });
    });
    return {
      aliases: ctor.aliases,
      allowUnknownOptions: ctor.allowUnknownOptions,
      allowVariadicParams: ctor.allowVariadicParams,
      categories: getInheritedCategories.getInheritedCategories(this),
      category: ctor.category,
      commands: this.commands,
      deprecated: ctor.deprecated,
      description: ctor.description,
      hidden: ctor.hidden,
      options,
      params: ctor.params,
      path: ctor.path,
      usage: ctor.usage
    };
  }
  /**
   * Return metadata as options for argument parsing.
   */


  getParserOptions() {
    const _this$getMetadata = this.getMetadata(),
          aliases = _this$getMetadata.aliases,
          allowUnknownOptions = _this$getMetadata.allowUnknownOptions,
          allowVariadicParams = _this$getMetadata.allowVariadicParams,
          options = _this$getMetadata.options,
          params = _this$getMetadata.params,
          path = _this$getMetadata.path;

    return {
      commands: [path, ...aliases],
      options,
      params,
      unknown: allowUnknownOptions,
      variadic: Boolean(allowVariadicParams)
    };
  }
  /**
   * Return the command path (canonical name on the command line).
   */


  getPath() {
    return getConstructor.getConstructor(this).path;
  }
  /**
   * Render a React element with Ink and output to the configured streams.
   */


  async render(element) {
    return this.getProgram().renderElement(element);
  }
  /**
   * Run the program within itself, by passing a custom command and argv list.
   */

  /**
   * Loop through class properties and register any options/params that are using
   * the initializer pattern, then validate all the metadata on the command.
   *
   * Note: We memoize so this only runs once!
   */
  // eslint-disable-next-line @typescript-eslint/member-ordering


  initializeAndValidate() {
    Object.entries(this).forEach(([prop, value]) => {
      if (common.isObject(value) && value[constants.INTERNAL_INITIALIZER]) {
        value.register(this, prop); // @ts-expect-error Allow this hack!

        this[prop] = value.value;
      }
    });
    const ctor = getConstructor.getConstructor(this);
    validateConfig.validateConfig(this.constructor.name, {
      aliases: ctor.aliases,
      allowUnknownOptions: ctor.allowUnknownOptions,
      allowVariadicParams: ctor.allowVariadicParams,
      categories: ctor.categories,
      category: ctor.category,
      deprecated: ctor.deprecated,
      description: ctor.description,
      hidden: ctor.hidden,
      path: ctor.path,
      usage: ctor.usage
    });
    validateOptions.validateOptions(ctor.options);
    validateParams.validateParams(ctor.params);
  }
  /**
   * Return the program instance or fail.
   */


  getProgram() {
    const program = this[constants.INTERNAL_PROGRAM];

    if (!program) {
      throw new CLIError.CLIError('COMMAND_NO_PROGRAM');
    }

    return program;
  }
  /**
   * Verify sub-command is prefixed with the correct path.
   */


}, _class2.aliases = [], _class2.allowUnknownOptions = false, _class2.allowVariadicParams = false, _class2.categories = {}, _class2.category = '', _class2.description = '', _class2.deprecated = false, _class2.hidden = false, _class2.options = globalOptions.globalOptions, _class2.params = [], _class2.path = '', _class2.usage = '', _temp), _rollupPluginBabelHelpers.applyDecoratedDescriptor(_class.prototype, "initializeAndValidate", [_dec], Object.getOwnPropertyDescriptor(_class.prototype, "initializeAndValidate"), _class.prototype), _class));
exports.Command = Command;
//# sourceMappingURL=Command.js.map
