'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const semver = require('semver');

const _interopDefault = e => e && e.__esModule ? e : {
  default: e
};

const semver__default = /*#__PURE__*/_interopDefault(semver);
/**
 * Verify that the currently running Node.js process.version satisfies the given semver range.
 * If not, a console error will be logged, or an error will be thrown.
 */


function checkNodeRequirement(range, throws = false) {
  return (argv, parse, logger) => {
    const _process = process,
          version = _process.version;

    if (!semver__default.default.satisfies(version, range)) {
      const message = `This tool requires a Node.js version compatible with ${range}, found ${version}.`;

      if (throws) {
        throw new Error(message);
      } else {
        logger.error(message);
      }
    }

    return parse(argv);
  };
}

exports.checkNodeRequirement = checkNodeRequirement;
//# sourceMappingURL=checkNodeRequirement.js.map
