'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const path = require('path');

const Path = require('./Path.js');

const _interopDefault = e => e && e.__esModule ? e : {
  default: e
};

const path__default = /*#__PURE__*/_interopDefault(path);
/**
 * An immutable class for operating on file system paths,
 * that always normalize using a forward slash ("/") for path separators.
 * Useful for paths found in configurations, globs, etc.
 */


class VirtualPath extends Path.Path {
  /**
   * Convert a path-like value to a formatted virtual path string.
   */
  static path(part) {
    return new VirtualPath(part).path();
  }
  /**
   * Return the current module path as a normalized string,
   * converting all path separators to "/".
   */


  path() {
    if (!this.isNormalized) {
      this.isNormalized = true;
      this.internalPath = path__default.default.normalize(this.internalPath).replace(/\\/g, '/');
    }

    return this.internalPath;
  }

}

exports.VirtualPath = VirtualPath;
//# sourceMappingURL=VirtualPath.js.map
