'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const isPlainObject = require('./isPlainObject.js');
/**
 * Can be used to recursively freeze plain objects with `Object.freeze`.
 *
 * ```ts
 * import { deepFreeze } from '@boost/common';
 *
 * const obj = deepFreeze({ foo: 123 });
 *
 * // Errors!
 * obj.foo = 456;
 * ```
 */


function deepFreeze(obj) {
  if (Object.isFrozen(obj)) {
    return obj;
  }

  const nextObj = {};
  Object.entries(obj).forEach(([key, value]) => {
    // Only freeze plain objects
    if (isPlainObject.isPlainObject(value, true)) {
      nextObj[key] = deepFreeze(value);
    } else {
      nextObj[key] = value;
    }
  });
  return Object.freeze(nextObj);
}

exports.deepFreeze = deepFreeze;
//# sourceMappingURL=deepFreeze.js.map
