'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const isObject = require('./isObject.js');
/**
 * Returns `true` if an object has no properties, an array has no items,
 * or the value is falsy, otherwise, it returns `false`.
 *
 * ```ts
 * import { isEmpty } from '@boost/common';
 *
 * isEmpty({}); // true
 * isEmpty({ name: 'Boost' }); // false
 *
 * isEmpty([]); // true
 * isEmpty(['Boost']); // false
 * ```
 */


function isEmpty(value) {
  return !value || Array.isArray(value) && value.length === 0 || isObject.isObject(value) && Object.keys(value).length === 0 || false;
}

exports.isEmpty = isEmpty;
//# sourceMappingURL=isEmpty.js.map
