import { PortablePath } from '@boost/common';
export declare class CrashReporter {
    contents: string;
    /**
     * Add a label with a value, or multiple values, to the last added section.
     */
    add(label: string, ...messages: (PortablePath | number | string)[]): this;
    /**
     * Start a new section with a title.
     */
    addSection(title: string): this;
    /**
     * Report Node.js related binary versions and paths.
     */
    reportBinaries(): this;
    /**
     * Report all environment variables.
     */
    reportEnvVars(): this;
    /**
     * Report common programming language versions and paths
     */
    reportLanguages(): this;
    /**
     * Report information about the current `process`.
     */
    reportProcess(): this;
    /**
     * Report the stack trace for a defined `Error`.
     */
    reportStackTrace(error: Error): this;
    /**
     * Report information about the platform and operating system.
     */
    reportSystem(): this;
    /**
     * Report npm package versions for all that match the defined pattern.
     * Only searches in the root node modules folder and _will not_ work with PnP.
     */
    reportPackageVersions(patterns: string[] | string, title?: string): this;
    /**
     * Write the reported content to the defined file path.
     */
    write(filePath: PortablePath): this;
}
//# sourceMappingURL=CrashReporter.d.ts.map