'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const BaseEvent = require('./BaseEvent.js');

const debug = require('./debug.js');

class BailEvent extends BaseEvent.BaseEvent {
  /**
   * Synchronously execute listeners with the defined arguments.
   * If a listener returns `false`, the loop with be aborted early,
   * and the emitter will return `true` (for bailed).
   */
  emit(args, scope) {
    if (process.env.NODE_ENV !== "production") {
      debug.debug('Emitting "%s%s" as bail', this.name, scope ? `:${scope}` : '');
    }

    return [...this.getListeners(scope)].some(listener => listener(...args) === false);
  }

}

exports.BailEvent = BailEvent;
//# sourceMappingURL=BailEvent.js.map
