'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const BaseEvent = require('./BaseEvent.js');

const debug = require('./debug.js');

class ConcurrentEvent extends BaseEvent.BaseEvent {
  /**
   * Asynchronously execute listeners for with the defined arguments.
   * Will return a promise with an array of each listener result.
   */
  async emit(args, scope) {
    if (process.env.NODE_ENV !== "production") {
      debug.debug('Emitting "%s%s" as concurrent', this.name, scope ? `:${scope}` : '');
    }

    return Promise.all([...this.getListeners(scope)].map(listener => listener(...args)));
  }

}

exports.ConcurrentEvent = ConcurrentEvent;
//# sourceMappingURL=ConcurrentEvent.js.map
