function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// Bundled with Packemon: https://packemon.dev
// Platform: node, Support: stable, Format: mjs
import fs from 'fs';
/* eslint-disable no-magic-numbers */

const COMPILER_OPTIONS = {
  allowJs: true,
  allowSyntheticDefaultImports: true,
  esModuleInterop: true,
  noEmit: true
};
const NODE_VERSION = Number.parseFloat(process.version.slice(1));

function isTypeScript(path) {
  return path.endsWith('.ts') || path.endsWith('.tsx');
}

function getModuleFormat(url) {
  if (url.endsWith('.json')) {
    return 'json';
  }

  return NODE_VERSION >= 12 ? 'module' : 'commonjs';
}

function getModuleFromNodeVersion(ts) {
  if (NODE_VERSION >= 15) {
    return ts.ModuleKind.ES2020;
  }

  if (NODE_VERSION >= 12) {
    return ts.ModuleKind.ES2015;
  }

  return ts.ModuleKind.CommonJS;
}

function getTargetFromNodeVersion(ts) {
  if (NODE_VERSION >= 16) {
    return ts.ScriptTarget.ES2020;
  }

  if (NODE_VERSION >= 15) {
    return ts.ScriptTarget.ES2019;
  }

  if (NODE_VERSION >= 14) {
    return ts.ScriptTarget.ES2018;
  }

  if (NODE_VERSION >= 13) {
    return ts.ScriptTarget.ES2017;
  }

  if (NODE_VERSION >= 12) {
    return ts.ScriptTarget.ES2016;
  }

  return ts.ScriptTarget.ES5;
} // https://nodejs.org/api/esm.html#esm_loaders


const FILE_WITH_EXT_PATTERN = /\.[a-z]+$/;

async function loadTypeScript() {
  try {
    return (await import('typescript')).default;
  } catch {
    return null;
  }
}

async function transform(url, source) {
  const ts = await loadTypeScript();

  if (!ts) {
    throw new Error(`\`typescript\` package required for transforming file "${url}".`);
  }

  return ts.transpileModule(String(source), {
    compilerOptions: _objectSpread(_objectSpread({}, COMPILER_OPTIONS), {}, {
      module: getModuleFromNodeVersion(ts),
      resolveJsonModule: false,
      target: getTargetFromNodeVersion(ts)
    }),
    fileName: url
  }).outputText;
} // NEW API


const resolve$1 = async (specifier, context, defaultResolve) => {
  if (isTypeScript(specifier)) {
    return {
      url: new URL(specifier, context.parentURL).href
    };
  } // Relative import doesn't have an extension, so attempt to find a TS file


  if (specifier.startsWith('.') && !FILE_WITH_EXT_PATTERN.test(specifier)) {
    for (const ext of ['.ts', '.tsx']) {
      const url = new URL(specifier + ext, context.parentURL); // @ts-expect-error Node not typed for URLs

      if (fs.existsSync(url)) {
        return {
          url: url.href
        };
      }
    }
  }

  return defaultResolve(specifier, context);
};

const load$1 = async (url, context, defaultLoad) => {
  if (isTypeScript(url)) {
    const format = getModuleFormat(url);

    if (format === 'commonjs') {
      return {
        format
      };
    }

    const _await$defaultLoad = await defaultLoad(url, {
      format
    }),
          rawSource = _await$defaultLoad.source;

    return {
      format,
      source: await transform(url, String(rawSource))
    };
  }

  return defaultLoad(url, context);
}; // OLD API


const getFormat$1 = async (url, context, defaultGetFormat) => {
  if (isTypeScript(url)) {
    return {
      format: 'module'
    };
  }

  return defaultGetFormat(url, context);
};

const transformSource$1 = async (source, context, defaultTransformSource) => {
  const url = context.url;

  if (isTypeScript(url)) {
    return {
      source: await transform(url, String(source))
    };
  }

  return defaultTransformSource(source, context);
};

const typescript = /*#__PURE__*/Object.freeze({
  __proto__: null,
  resolve: resolve$1,
  load: load$1,
  getFormat: getFormat$1,
  transformSource: transformSource$1
}); // Once we have multiple loaders, we can iterate through them as a whole.

const getFormat = typescript.getFormat,
      load = typescript.load,
      resolve = typescript.resolve,
      transformSource = typescript.transformSource;
export { getFormat, load, resolve, transformSource, typescript };
//# sourceMappingURL=loader.mjs.map
