'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const debug = require('./debug.js');

const ParallelPipeline = require('./ParallelPipeline.js');

class ConcurrentPipeline extends ParallelPipeline.ParallelPipeline {
  /**
   * Execute all work units in parallel with a value being passed to each work unit.
   * If an error occurs, the pipeline will abort early, otherwise return a list of all results.
   */
  async run() {
    const context = this.context,
          value = this.value;
    const work = this.getWorkUnits();
    debug.debug('Running %d in parallel', work.length);
    this.onBeforeRun.emit([value]);
    const result = await Promise.all(work.map(unit => {
      this.onRunWorkUnit.emit([unit, value]);
      return unit.run(context, value);
    }));
    this.onAfterRun.emit([]);
    return result;
  }

}

exports.ConcurrentPipeline = ConcurrentPipeline;
//# sourceMappingURL=ConcurrentPipeline.js.map
