'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const debug = require('./debug.js');

const SerialPipeline = require('./SerialPipeline.js');
/* eslint-disable no-await-in-loop */


class WaterfallPipeline extends SerialPipeline.SerialPipeline {
  /**
   * Execute the pipeline in sequential order with the output of each
   * work unit being passed to the next work unit in the chain.
   */
  async run() {
    const work = this.getWorkUnits();
    let value = this.root.value;
    debug.debug('Running %d as a waterfall', work.length);
    this.onBeforeRun.emit([value]);

    for (const unit of work) {
      this.onRunWorkUnit.emit([unit, value]);
      value = await unit.run(this.context, value);
    }

    this.onAfterRun.emit([]);
    return value;
  }

}

exports.WaterfallPipeline = WaterfallPipeline;
//# sourceMappingURL=WaterfallPipeline.js.map
