'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const PipelineError = require('./PipelineError.js');

const Task = require('./Task.js');

const WorkUnit = require('./WorkUnit.js');
/**
 * Create or return an executable work unit. Supports the following patterns:
 *
 * - A `Routine` instance.
 * - A `Task` instance.
 * - A custom `WorkUnit` instance.
 * - A title and function that returns a `Task` instance.
 */


function createWorkUnit(titleOrWorkUnit, // eslint-disable-next-line @typescript-eslint/no-explicit-any
action, scope) {
  if (titleOrWorkUnit instanceof WorkUnit.WorkUnit) {
    return titleOrWorkUnit;
  }

  if (typeof titleOrWorkUnit === 'string' && typeof action === 'function') {
    return new Task.Task(titleOrWorkUnit, scope ? action.bind(scope) : action);
  }

  throw new PipelineError.PipelineError('WORK_UNKNOWN');
}

exports.createWorkUnit = createWorkUnit;
//# sourceMappingURL=createWorkUnit.js.map
