import { FilePath, ModuleID, ModuleResolver } from '@boost/common';
export declare type Source = FilePath | ModuleID;
export declare type SourceOptions = boolean | object;
export declare type SourceWithOptions = [Source, SourceOptions];
export declare type Callback<T = unknown> = (value: T) => Promise<void> | void;
export interface Pluggable<T = any> {
    /** Unique name of the plugin. Typically the npm package name. */
    readonly name: ModuleID;
    /** Priority in which to order the plugin. */
    priority?: number;
    /** Life cycle called when the plugin is unregistered. */
    shutdown?: Callback<T>;
    /** Life cycle called when the plugin is registered. */
    startup?: Callback<T>;
}
export declare type Setting = Record<string, SourceOptions>;
export declare type Factory<T extends Pluggable, O extends object = object> = (options: Partial<O>) => Promise<T> | T;
export interface RegisterOptions<T = unknown> {
    /** Override the priority of the plugin. */
    priority?: number;
    /** Custom tool instance to pass to life cycles. */
    tool?: T;
}
export interface Registration<T extends Pluggable> extends RegisterOptions {
    /** Unique name of the plugin. */
    name: ModuleID;
    /** Plugin instance or object. */
    plugin: T;
}
export interface RegistryOptions<T extends Pluggable> {
    /** Callback fired after a plugin's `shutdown` life cycle is executed. */
    afterShutdown?: Callback<T>;
    /** Callback fired after a plugin's `startup` life cycle is executed. */
    afterStartup?: Callback<T>;
    /** Callback fired before a plugin's `shutdown` life cycle is executed. */
    beforeShutdown?: Callback<T>;
    /** Callback fired before a plugin's `startup` life cycle is executed. */
    beforeStartup?: Callback<T>;
    /** Custom module resolver. */
    resolver?: ModuleResolver;
    /** Validate the shape of the plugin being registered. */
    validate: Callback<T>;
}
//# sourceMappingURL=types.d.ts.map