"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const zx_1 = require("zx");
const core_1 = require("@beemo/core");
class ReleaseScript extends core_1.Script {
    name = 'release';
    parse() {
        return {
            options: {
                dryRun: {
                    description: 'Execute a dry run',
                    type: 'boolean',
                },
            },
        };
    }
    async execute(context, args) {
        const { GITHUB_TOKEN, GITHUB_REPOSITORY } = process.env;
        if (!GITHUB_REPOSITORY) {
            throw new Error('Release step must be run inside of a GitHub Action');
        }
        const origin = `https://x-access-token:${GITHUB_TOKEN}@github.com/${GITHUB_REPOSITORY}.git`;
        await (0, zx_1.$) `npm config set workspaces-update false`;
        await (0, zx_1.$) `git remote set-url origin ${origin}`;
        await (0, zx_1.$) `git config --global user.email "action@github.com"`;
        await (0, zx_1.$) `git config --global user.name "GitHub Action"`;
        const dryRun = args.options.dryRun ? '--dry-run' : '';
        await (context.workspaces.length > 0
            ? (0, zx_1.$) `yarn rrun multi-semantic-release ${dryRun}`
            : (0, zx_1.$) `yarn rrun semantic-release --tag-format '\${version}' ${dryRun}`);
    }
}
exports.default = () => new ReleaseScript();
//# sourceMappingURL=Release.js.map